function yesNo = isBidirectionalAndSymmetric(linkTable, weightPerLink)

    % Check whether the topology is bidirectional and symmetric: same number
	% of links between each node pair in both directions (assuming
	% multi-digraphs) and same weights per direction
	%
	% A weight can be: link capacity, link length ...
    %
    % Syntax: yesNo = isBidirectionalAndSymmetric(linkTable, weightPerLink)
    %
    % Input parameters:
    % - linkTable (Ex2): A table in which each row represents
	%   the origin and the destination nodes of the link
    % - weightPerLink (1xE): Row vector with a weight per link
    %
    % Output parameters:
    % - yesNo: '1' if is traffic-symmetric, '0' if not
	
	%% Sanity checks
	assert(isLinkTable(linkTable), 'A valid set of links is required');
	E = size(linkTable, 1);
	assert(isequal(size(weightPerLink), [1 E]) && all(isfinite(weightPerLink)), 'weightPerLink must be a 1xE vector');
	
	%% Perform check
	yesNo = 0;

	uniqueLinkTable = unique(linkTable, 'rows');
	upwardLinks = find(diff(uniqueLinkTable, 1, 2) > 0);
	downwardLinks = find(diff(uniqueLinkTable, 1, 2) < 0);
	
	if numel(upwardLinks) ~= numel(downwardLinks), return; end
	
	for linkId = 1:numel(upwardLinks)
		
		setOfUpwardLinks = find(ismember(linkTable, uniqueLinkTable(linkId,:), 'rows'));
		setOfDownwardLinks = find(ismember(linkTable, uniqueLinkTable(linkId,[2 1]), 'rows'));
		
		if numel(setOfUpwardLinks) ~= numel(setOfDownwardLinks), return; end
		
		if ~isequal(sort(weightPerLink(setOfUpwardLinks)), ...
				sort(weightPerLink(setOfDownwardLinks)))
		   return; 
		end
		
	end
	
	yesNo = 1;

end