function [costMatrix nextHopMatrix] = floydAlgorithm(linkTable, costPerLink, N)

	% Perform Floyd's algorithm [1] for finding shortest path in a weighted graph
	% with positive weights.
	%
	% Input parameters:
	% - linkTable (Ex2): Table in which each row represents the origin and destination
	%   node of a link
	% - costPerLink (1xE): Positive weight of each link
	% - N: Number of nodes in the whole network
	%
	% Output parameters:
	% - costMatrix (NxN): Each entry (i,j) represents the minimum cost to go from i to j
	% - nextHopMatrix (NxN): Each entry (i,j) represents the next hop to go from i to j
	%
	% References
	%
	% [1] R.W. Floyd, "Algorithm 97: Shortest Path", Communications of the ACM
	% vol. 5, no. 6, pp. 345-345, June 1962
	
	%% Sanity checks
	assert(numel(N)==1 && isfinite(N) && N==round(N) && N > 0, 'N must be greater than 0');
	assert(isLinkTable(linkTable, N), 'A valid set of links is required');
	assert(~isempty(costPerLink) && ndims(costPerLink) == 2 && ...
			isequal(size(costPerLink), [1 E]) && all(costPerLink >= 0) && ...
			all(isfinite(costPerLink(:))), 'costPerLink must be a non-negative 1xE vector');
			
	costMatrix = accumarray(linkTable, costPerLink, [N N], @min, inf);
	costMatrix(sub2ind([N N], 1:N, 1:N)) = 0;

	if nargout > 1
		%% Initialize nextHopMatrix
		nextHopMatrix = kron(1:N,ones(N,1));
		nextHopMatrix(sub2ind([N N], 1:N, 1:N)) = 0;
		nextHopMatrix(costMatrix == inf) = 0;
		
		for intermediateNode = 1:N
			newCostMatrix = min(costMatrix, ...
				repmat(costMatrix(:,intermediateNode), [1 N]) + ... 
				repmat(costMatrix(intermediateNode,:), [N 1]));
			
			dontTouch = (newCostMatrix == costMatrix);
			dontTouch(intermediateNode, :) = true;
			dontTouch(:, intermediateNode) = true;

			costMatrix = newCostMatrix;
			[row column] = find(~dontTouch);
			nextHopMatrix(sub2ind([N N], row, column)) = nextHopMatrix(row, intermediateNode);
		end
	else
		for intermediateNode = 1:N
			costMatrix = min(costMatrix, ...
				repmat(costMatrix(:,intermediateNode), [1 N]) + ... 
				repmat(costMatrix(intermediateNode,:), [N 1]));
		end
	end
    
end
